function [lamk, out] = gettheta(x, y, G0, c0, b0, K1, K0, lamk, M, theta, maxit, flag_theta0, reltol)

% Given the data, lambda0, and M, solve for the estimate. G0, c0, b0 is the starting G, c, b. 

n = length(y);
d=size(x,2);
D = size(G0,2);
if D==(d * (d+1)/2), 
  ord=2;
else
  ord=1;
end

np=d;
np2=D-d;

lam=1;
if isreal(lamk)
  if length(lamk)~=D ,
    if length(lamk)>1,
      error('wrong dimension for  lam')
      return
    else
      lamk=lamk([ones(np2,1) ones(np2,1)*2]);
    end
  end
  lamk1=lamk(1:np);
  if D>np,
    lamk2=lamk(np+1:end);
    lamk2=[lamk2(:) lamk2(:) lamk2(:) lamk2(:)];
  end
elseif isstruct(lamk),
  lamk1=lamk.lamk1;
  if length(lamk1)~=np ,
    if length(lamk1)>1,
      error('wrong dimension for  lamk1')
      return
    else
      lamk1=lamk1(ones(np,2));
    end
  end

  lamk2=lamk.lamk2;
  if size(lamk2,1)~=np2 ,
    if length(lamk2)>1,
      error('wrong dimension for  lamk2')
      return
    else
      lamk2=lamk2([ones(np2,1) ones(np2,1)*2 ones(np2,1)*3 ones(np2,1)*4]);
    end
  end
  
else
  error('wrong variable type for lam')


end


if nargin<10 | isempty(theta),
  theta=ones(D,1);
end
if nargin<11 | isempty(maxit),
  maxit=2;
end
if nargin<12 | isempty(flag_theta0),
  flag_theta0=0;
end
if nargin<13 | isempty(reltol),
  reltol=1.e-2;
end

iter=0;
relnorm=1;
indth0=find(theta<1.e-9);
indth=find(~(theta<1.e-9));

while (iter<maxit & relnorm>= reltol),
  iter=iter+1;
  ycb0 = y - c0*n /2 - b0;
  theta0=theta;
  
  
  warning off;
  if M==0,
    theta=theta*0;
  else
    opts=optimset('lsqlin');
    opts.Display='off';
    if norm(G0(:,indth),'inf')==0,
      theta(indth)=0;
    else
      theta(indth) = lsqlin(G0(:,indth), ycb0, ones(1,length(indth)), M, [], [], zeros(length(indth),1),[],theta0(indth)*flag_theta0,opts);
    end
  end
  indth0=find(theta<1.e-9);
  indth=find(~(theta<1.e-9));
  theta(indth0)=0;
    
  [yt2, out] = tcubic_spl_cosso(x, y, K1, K0, lamk, ord, theta);
  if ord==2,
    [G0, yfit]=tcubic_anova_cosso(x, out, K1, K0, sign(theta(1:d)), lamk1, sign(theta(d+1:end)), lamk2); 
    G0=[G0, yfit];
    clear yfit,
  else
    G0=tcubic_anova_cosso(x, out, K1, K0, sign(theta), lamk1); 
  end
  b0=out.d(1);
  c0=out.c;
%     Now check for convergence
      divisor = theta;
      divisor(theta < 1E-6) = 1;
      relnorm = sqrt(sum(((theta0-theta)./divisor).^2));
      
  
end


if nargout>1,
  
  for inneriter = 1:d
    f(:, inneriter) = G0(:,inneriter) *theta(inneriter);
    L2(inneriter) = sqrt(mean(f(:,inneriter).^2));
    H2(inneriter)=sqrt(c0'*f(:,inneriter));
  end
  if ord==2,
    index = d;
    for i = 1:(d-1)
      for j = (i+1):d
        index = index + 1;
        f(:, index) = G0(:,index) * theta(index);
        L2(index) = sqrt(mean(f(:,index).^2));
        H2(index)=sqrt(c0'*f(:,index));
      end
    end
  end  
  
  out.theta = theta;
  out.M = M;
  
  out.H2 = H2;
  out.L2 = L2;
  
  
  
end

