function [gsa2, gsa_] = gsa_sdp_spl_2nd(y2, x0, gsa_, icomp, indx, opts, nvr, chk, yi_anal, fname)

% [gsa2, gsa_] = 
% gsa_sdp_spl_2nd(y2, x0, gsa_, icomp, indx, opts, nvr, chk, yi_anal, fname)
%                 1   2   3     4      5     6     7    8    9        10      
%                   
% inputs: 
% y2: model output
% x0: original input sample
% gsa_: first order structure from gsa_sdp 
% icomp:
%        0 = plot previous analysis
%        1 = optimize lambdas for interactions using RW
%        2 = optimize lambdas for interactions using IRW
%        -1 = fixed lamdas (nvr's) specified by user using RW
%        -2 = fixed lamdas (nvr's) specified by user using IRW
% indx: second order indices to analyse ([] or -1 default)
%       if length(indx)==1, compute ALL 2nd order effects for input factor indx
%       if length(indx)>1, compute effect WITHIN the group of factors indx
%       default: all 2nd order indices
% opts: estimation options [nback tcrit nest nsax tstcrit irank ialias lper bkp_opt], use -1 for defaults
%                           
%       nback: number of backfitting iterations (2)
%       tcrit: convergence criterion of backfitting (0.001)
%       nest: number of backfitting loops with ML estimation (2)
%       nsax: number of samples actually used for optimization and for the basis
%          dimension: [ min(256, length(y)) is default ]
%       tscrit: critical value for tstat for regressor selection
%       irank: = 1 to rank transform x data (default)
%              = 0 do transformation: NEEDS uniform inputs [0 1]!!!!
%       ialias: specify if some main effects have NOT to be taken out from y 
%              (for lptau samples < 1024 runs, column 14 can have aliasing )
%       lper: number of sample in the semi-period for sort2 (16)
%       bkp_opt: keep (=1) or delete (=0) intermediate bkp files saved at
%              each backfitting iteration (0)
% nvr:  nvr values for 2nd order interactions
% chk:  binary array for inclusion of interactions
% yi_anal: if analytic results are available, for comparisons
% fname: assign a name to the files saved by the routine (gsa_sdp)
%
% uses sdr2UU: for univariate estimates of joint effects
%      sdr2it0: for each backfitting iteration of joint effects
%      fast2hdmr: to construct the interaction as the product of two zero
%      mean functions

if nargin==0,
  disp(' ')
  disp('[s2, s22, v2, yfit, yfitse] = gsa_sdp_2nd(y2, x0, f, icomp, indx, opts, nvr, yi_anal, fname)')
  disp('                                          1   2   3  4      5     6     7    8        9     ')
  disp(' ')
  return
end

if nargin<4,
  disp('Insufficient number of inputs')
  return
end

if icomp==0,
  gsa2=gsa_{2};
  gsa_=gsa_{1};
end

if iscell(gsa_),
  out=gsa_{2};
  gsa_=gsa_{1};
end

kn=length(y2);
np=size(x0,2);

if nargin<6 | isempty(opts)
    opts = [2 0.001 2 min(kn,256) 1 1 0 16 0];
end
opts0=[2 0.001 2 min(kn,256) 1 1 0 16 0];
if length(opts)<9;
    opts=[opts opts0(length(opts)+1:9)];
end

opts(find(opts<0))=opts0(find(opts<0));

nvr1=gsa_.nvr;
nback=opts(1);
tcrit=opts(2);
nest=opts(3);
nsax=opts(4);
tscrit=opts(5);
irank=opts(6);
ialias=opts(7);
lper=opts(8);
bkp_opt=opts(9);

global tstat2crit
tstat2crit=tscrit;

f = gsa_.f.*0;
itest=find(gsa_.stat>tstat2crit);
f(:,itest)=gsa_.f(:,itest);

iok=find(~isnan(y2));
inok=find(isnan(y2));
my=mean(y2(iok));
y0=y2;
y2=y2-my;

%[y(find(~isnan(y))),my,sy]=stand_(y(find(~isnan(y))));
if nargin<10 | isempty(fname)
    fname='gsa_sdp';
end

if irank,
  x=x0*0;
  x=trank(x0)-1/(2*length(y2));
%   x(iok,:)=trank(x0(iok,:))-1/(2*length(iok));
%   if ~isempty(inok),
%     for j=1:np,
%       if max(x0(inok,j))>max(x0(iok,j))
%         xout1=max(x0(inok,j));
%         xxout1=1;
%       else
%         xout1=[];
%         xxout1=[];
%       end
%       if min(x0(inok,j))<min(x0(iok,j))
%         xout0=min(x0(inok,j));
%         xxout0=0;
%       else
%         xout0=[];
%         xxout0=[];
%       end
%       
%       
%       x(inok,j)=interp1([x0(iok,j); xout0; xout1],[x(iok,j); xxout0; xxout1],x0(inok,j));
%     end
%   end
else
  if max(max(x0))>1 | min(min(x0))<0,
    error('irank=0: Inputs MUST be in the unit hypercube!')
  end
  x=x0;

end
% clear x0

if isempty(f),
  f=x*0;
end

if nargin<5 | isempty(indx)
  indx = [1:np];
end
indx=sort(indx);
if max(indx)>np,
  disp('Some indices exceeded number of parameters!!!!')
  indx=indx(find(ismember(indx,[1:np])));  % check that no index is outside bounds
end

sy=std(y2(iok));

if length(indx)>1,
  i2 = nchoosek(indx,2);
  jj2=nchoosek(length(indx),2);
else
  i2=indx(ones(np-1,1));
  i2(:,2)=[1:indx-1,indx+1:np]';
  jj2=np-1;
end

% if jj2==1;
%   nest=1;
%   nback=0;
% end


if icomp, %| isempty(dir([fname,'_2nd.mat']))
%     if isempty(dir([fname,'_2nd.mat']))
%         icomp=1;
%     end
    
%     y0=y2;
    for j=1:np,
      if ~(j==ialias & ialias)
        y2=y2-f(:,j);
      end
    end

    s2=zeros(jj2,2);
    v2=s2;
    s22=zeros(np,np);
    yfit=zeros(kn , jj2);
    if exist('out'),
      yfit=out.f(:,np+1:end);
      s2(:,1)=var(yfit)./var(y0);
    end
    save([fname,'_temp'],'s2','v2','s22')
    save([fname,'_yfit'],'yfit')

    if nargin<7 | isempty(nvr),
      nvr0=( 2*sin( 0.5*(2*pi/(8*lper)) ) )^4; % modified 3/3/2007
      nvr=ones(1,jj2).*nvr0;
    elseif length(nvr)==1,
      nvr=ones(1,jj2).*nvr;
    elseif length(nvr)~=jj2,
      error('Wrong dimension of input NVR!')
    end

    diary on,
    tic,



    imult=[];
    ibkp=0;
    if nest
      if nargin<8 | isempty(chk),
        chk=zeros(size(i2,1),1);
      end
    else
      if nargin<8 | isempty(chk),
        chk=ones(size(i2,1),1);
      end
    end
    if length(chk)==1,
      chk=chk(ones(jj2,1));
      chk=chk(:);
    end

    % parametric terms
    %           S=[ones(size(y2)) x-0.5];
    %           clear combs0;
    %           for j=1:nchoosek(np,2),
    %             [i2x]=combs0([1:np],2);
    %             S=[S (x(:,i2x(1))-0.5).*(x(:,i2x(2))-0.5)];
    %           end
    %           par0=S\y2;
    %           for j=1:nchoosek(np,2),
    %             yfit(:,j)=S(:,j+1+np).*par0(j+1+np);
    %           end
    %           s2=var(yfit)'./var(y0);
    %           imult=find(s2>2/kn);
    %           chk(imult)=1;
    %           clear par0 S
%     [fit, yfit, s2, s22, v2, nvr, chk, gsa_, outk2] = ...
%       sdr2AA_spl(y2, y0, x, lper, i2, nvr, chk, gsa_, nest, tcrit, sy, ibkp, fname);
    if exist('out'),
      [fit, yfit, s2, s22, v2, nvr, chk, gsa_, outk2] = ...
        sdr2_cosso3(y2, y0, x,  abs(icomp)-1,  i2, nvr, chk, {gsa_, out}, nest, tcrit, sy, ibkp, fname);
    else
      [fit, yfit, s2, s22, v2, nvr, chk, gsa_, outk2] = ...
        sdr2_cosso3(y2, y0, x,  abs(icomp)-1,  i2, nvr, chk, gsa_, nest, tcrit, sy, ibkp, fname);
    end      
%       sdr2_cosso2(y2, y0, x,       i2, nvr, chk, gsa_, nest, tcrit, sy, ibkp, fname);
%       sdr2_cosso(y2, y0, x,       i2, nvr, chk, gsa_, nest, tcrit, sy, ibkp, fname);
%       sdr2JJ_spl(y2, y0, x,       i2, nvr, chk, gsa_, nest, tcrit, sy, ibkp, fname);
    gsa_.f0=my;
    gsa_.x0=x0;

    disp('Second order interactions completed')
    toc,




   
%     if ~bkp_opt,
      delete([fname,'_2nd_bkp_*.mat']);
%     end
%     load( [fname,'_yfit'])
%     load([fname,'_2nd.mat'],'yfitse')
%     delete([fname,'_yfit.mat'])
    
  rsq=1-cov(fit-y0)/cov(y0);
  gsa2.fit=fit;
  gsa2.f=yfit;
%   gsa2.fse=yfitse;
  gsa2.nvr=nvr;
  gsa2.r2=rsq;
  gsa2.s22=s22;
  gsa2.s2=s2(:,1);
  gsa2.s2_std=s2(:,2);
  gsa2.stat=s2(:,3);
  gsa2.chk=chk;
  gsa2.out=outk2;
  gsa2.indx=indx;
  gsa2.i2=i2;
    
    diary off
else
  indx0=indx;
  i20=i2;
  yfit=gsa2.f;
%   yfitse=[];
  chk=gsa2.chk;
%   v2=[];
  s2=gsa2.s2;
  s22=gsa2.s22;
  i2=gsa2.i2;
  indx=gsa2.indx;
%     load([fname,'_2nd'],'yfit','yfitse','chk','v2','s2','s22','i2');
%     if isempty(who('-file',[fname,'_2nd'],'indx'))
%       indx=[1:np];
%     else
%       load([fname,'_2nd'],'indx');
%     end
    i2str=num2str(i2);
    i20str=num2str(i20);
    
%     if length(indx0)>2,
%       indx0=indx0(find(ismember(indx0,indx))); % check that indx contains indx0
%       i20 = nchoosek(indx0,2);
%       jj2=nchoosek(length(indx0),2);
% 
%     end

    figure,
    
    npl=length(indx0);
    if npl==1,
      npl=ceil(sqrt(jj2));
    else
      ix2=nchoosek([1:npl],2);
    end
    
    for j=1:jj2,
        j1=min(i20(j,:));
        j2=max(i20(j,:));
        if length(indx0)>1,
        subplot(npl,npl,npl*(ix2(j,1)-1)+ix2(j,2))
        else
        subplot(npl,npl,j)
        end
        a1=find(i2(:,1)==j1(ones(size(i2,1),1)));
        a2=find(i2(a1,2)==j2(ones(length(a1),1)));

        if ~isempty(a2)
        ccontour(x(:,j1), x(:,j2), yfit(:,a1(a2)));

        xa=get(gca,'xlim');
        ya=get(gca,'ylim');
        text(0.1,1.1,num2str(s2(a1(a2)),'%5.3f'),'units','normalized')
        text(0.4,1.1,['f_{',int2str(j1),'}_{',int2str(j2),'}'],'units','normalized')
        end
    end
    if length(indx0)>1,

    for j=1:npl,
        subplot(npl,npl,npl*(j-1)+j)
        plot(x(:,indx0(j)), [f(:,indx0(j))],'.r');
        %title(num2str(var(f(:,j))./sy^2,'%4.2f'))
        xa=get(gca,'xlim');
        ya=get(gca,'ylim');
        text(0.1,0.5,num2str(var(f(:,indx0(j)))./sy^2,'%5.3f'),'units','normalized')
        text(0.4,1.1,['f_{',int2str(indx0(j)),'}'],'units','normalized')
    end
    
    end
    saveas(gcf,[fname,'_2nd'])    
end