function yf = ss_anova_fcast(x, gsa1, gsa2, igraph),
%
% copyright Marco Ratto 2008, Joint Research Centre
%
% yf = ss_anova_fcast(x, gsa1, gsa2),
%
% Prediction engine for tensor product cubic spline
%
% x : new input data set
% gsa1: first order info from ss_anova
% gsa2 (optional): second order info from ss_anova
%
% with 2 input arguments provides forecast using first order model
% with 3 input arguments provides forecast using second order model
%

if nargin==0,
  disp(' ')
  disp('yf = ss_anova_fcast(x, gsa1, gsa2)')
  disp(' ')
end

if nargin<4 | isempty(igraph),
  igraph=0;
end

x0=gsa1.xx;
[kn, np] = size(x0);
if max(max(x0-gsa1.x0))~=0,
  for j=1:np,
    x0dum=gsa1.x0(:,j);
    xxdum=gsa1.xx(:,j);
    if max(x(:,j))>max(x0dum),
      dx=(max(x(:,j))-max(x0dum))/(max(x0dum)-min(x0dum))/1.1;
      x0dum=[max(x(:,j)); x0dum];
      
      xxdum=[min(1, dx+max(xxdum)); xxdum];
    end
    if min(x(:,j))<min(x0dum),
      dx=(-min(x(:,j))+min(x0dum))/(max(x0dum)-min(x0dum))/1.1;
      x0dum=[min(x(:,j)); x0dum];
      xxdum=[max(0, min(xxdum)-dx); xxdum];
    end    
    x(:,j) = interp1(x0dum,xxdum,x(:,j));
  end
end

if nargin==2,
  disp('Prediction from first order meta-model')
  [Q, S] = tcubic_kdm(x,x0,1,[gsa1.out.chk],1./[gsa1.out.nvr]./kn^4);
  yf=S*gsa1.out.d+Q*gsa1.out.c;
else
  disp('Prediction from second order meta-model')
  if min(size(gsa2.nvr)==1),
    [Q, S] = tcubic_kdm(x,x0,2,[gsa1.chk; gsa2.chk],1./[gsa1.nvr gsa2.nvr]./kn^4);
  else
K03d = zeros(size(x,1),kn,np);
K13d = zeros(size(x,1),kn,np);
for i = 1:np,
  [H, H0] = cubic_dm(x0(:,i),x(:,i));
  K13d(:,:,i) = H;
  K03d(:,:,i) = H0;
end
%     lams.lamk1=1./gsa1.nvr./kn^4;
%     lams.lamk2=(1./gsa2.nvr./kn^4)';
%     [Q, S] = tcubic_kdm(x,x0,2,[gsa1.chk; gsa2.chk],lams);
    Q = tcubic_kdm_cosso(K13d, K03d, gsa2.out.lams, 2, gsa2.out.theta);
    S=ones(size(x,1),1);
  end
  yf=S*gsa2.out.d+Q*gsa2.out.c;
  if igraph
    gsa_sdp_spl_2nd(gsa1.y, gsa1.x0, {gsa1, gsa2}, 0);
  end
end


