function [yt, out] = tcubic_spl_cosso(X, y, K1, K0, lamk, ord, theta);
% solves tensor product cubic spline for COSSO, i.e. putting parametric
% terms into Q

if nargin<5,
  disp(' ')
  disp('[yt, out] = tcubic_spl_cosso(X, y, K1, K0, lamk, ord, theta);')
  disp(' ')
end

if nargin<6 | isempty(ord), ord=1; end
if ord<1 | ord>2,
  error('ord can be 1 or 2!')
end

n = length(y);
np=size(X,2);

if ord==1,
  nterm=np;
else
  np2=nchoosek(np,2);
  nterm=np+np2;
  j2=nchoosek([1:np],2);
end

if nargin<7 | isempty(theta), theta=ones(nterm,1); end
chk=sign(theta);

if length(chk)==1, 
  chk=chk*ones(nterm,1);
end
if ord==1,
  chk1=chk;
else
  chk1=chk(1:np);
  chk2=chk(np+1:end);
end
lam=1;
% lam0=1/(n*1.e5);
if isreal(lamk)
  if length(lamk)~=nterm ,
    if length(lamk)>1,
      error('wrong dimension for  lam')
      return
    else
      lamk=lamk(ones(nterm,1));
    end
  end
  lamk1=[lamk(1:np) lamk(1:np)];
  if nterm>np,
    lamk2=lamk(np+1:end);
    lamk2=[lamk2(:) lamk2(:) lamk2(:) lamk2(:)];
  end
elseif isstruct(lamk),
  lamk1=lamk.lamk1;
  if length(lamk1)~=np ,
    if length(lamk1)>1,
      error('wrong dimension for  lamk1')
      return
    else
      lamk1=lamk1(ones(np,2));
    end
  end

  lamk2=lamk.lamk2;
  if size(lamk2,1)~=np2 ,
    if length(lamk2)>1,
      error('wrong dimension for  lamk2')
      return
    else
      lamk2=lamk2([ones(np2,1) ones(np2,1)*2 ones(np2,1)*3 ones(np2,1)*4]);
    end
  end
  
else
  error('wrong variable type for lam')


end

Q=zeros(n,n);
S=ones(n,1);
for j=1:np,
  if chk1(j),
    Q = Q+(K0(:,:,j)./lamk1(j,1)+K1(:,:,j)./lamk1(j,2)).*theta(j);
  end
end
if ord==2,
for j=1:np2,
  if chk2(j),
%     [H, dum, H0]=tensor_cubic_dm(X(:,j2(j,:)),[],lamk2(j,2:4));
%     H0=H0./lamk2(j,1);
%     Q = Q+(H0./lam0*100+H).*theta(j+np);
    ind1=j2(j,1);
    ind2=j2(j,2);
    Q = Q + (K0(:,:,ind1).*K0(:,:,ind2)./lamk2(j,1) + ...
        K1(:,:,ind1).*K1(:,:,ind2)./lamk2(j,2) + ...
        K1(:,:,ind1).*K0(:,:,ind2)./lamk2(j,3) + ...
        K0(:,:,ind1).*K1(:,:,ind2)./lamk2(j,4) ).*theta(j+np);
    
  end
end
end

[yt, out] = splfun(y,1,S,Q);

out.theta=theta;